C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod28_write_products(out_handle,hdr_lun,scan,nscans)

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Write MOD28 retrieval products and QA data to the output file.
c
c!Input Parameters:
c    OUT_HANDLE    LUN for MOD28 output file
c    HDR_LUN       LUN for product header output file
c    SCAN          Scan number within L1B granule
c    NSCANS        Total number of scans in this data set
c
c!Output Parameters:
c    None
c
c    The following arrays in COMMON /MOD28_DATA/ are written to output:
c    SST                   Two versions
c    RAW_RADIANCE          Raw radiance x 5 channels
c    BRIGHTNESS_TEMP       Brightness temperature x 5 channels
c    DAY_NIGHT_FLAG        *** REMOVE AFTER TESTING ***
c    LAND
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------
      
      implicit none


      include 'db_mod28uw_data.inc'
      
c ... arguments

      integer out_handle, scan, nscans, nboxes, hdr_lun

c ... local variables

      integer i, j,  k

      save / mod28_data / 

c ... local arrays
      real array(max_pixel,max_line)
      character*2 band_no
      character*3 b_unit

c ... data statments

c ... The order of bands is:
c     1.  SST         (deg C)
c     2.  SST4        (deg C)
c     3.  radiance_20 (W/m2/um/sr)
c     4.  radiance_22 (W/m2/um/sr)
c     5.  radiance_23 (W/m2/um/sr)
c     6.  radiance_31 (W/m2/um/sr)
c     7.  radiance_32 (W/m2/um/sr)
c     8.  bright_20   (K)
c     9.  bright_22   (K)
c     10. bright_23   (K)
c     11. bright_31   (K)
c     12. bright_32   (K)

c ... SST fields

      call write_output( out_handle, hdr_lun, scan, nscans, 1,
     &sst1,'SST','C')

      call write_output( out_handle, hdr_lun, scan, nscans, 2,
     &sst4,'SST4','C')

c ... raw radiances

       do k = 1 , n_output
        write(band_no,'(I2)') sstband(k)
        call write_output(out_handle, hdr_lun, scan, nscans, k+2, 
     &  raw_radiance(1,1,k), 'Raw_Radiance_B'//band_no,'Rad')
      end do

c ... brightness temperatures

c      b_unit = 'tmp'
      do k = 1 , n_output
        write(band_no,'(I2)') sstband(k)
        call write_output(out_handle,hdr_lun,scan,nscans,k+n_output+2,
     &  brightness_temp(1,1,k),'Brightness_Temperature_B'// band_no ,
     &  'K')
      end do
                 
      END

c-----------------------------------------------------------------------

      subroutine write_output(out_handle,hdr_lun,scan,nscans,
     &                        req_band,array,arrnm,b_unit_name)
c-----------------------------------------------------------------------
c!F77
c
c!Description:
c    Write MOD28 retrieval products to the output file.
c
c!Input Parameters:
c    OUT_HANDLE    LUN for MOD28 output file
c    HDR_LUN       LUN for product header output file
c    SCAN          MODIS scan number
c    NSCANS        Total number of scans in this data set
c    REQ_BAND      Output file band number
c    ARRAY         Array of product values
c                  (Bad values should be set to BAD_VALUE in the calling
c                  routine: see mod28uw_data.inc for BAD_VALUE value)
c    ARRNM         Name of the array (SDS name) in the output file
c    b_unit_name   textual name of band for header file
c
c!Output Parameters:
c    None
c
c!Revision History:
c    12-DEC-1997 Kathy Strabala 
c                Changed BYTE to INTEGER*1 so that mapi would 
c                like it
c    10-DEC-1997 Liam Gumley
c                Added BYTE data types
c                Now gets _FillValue directly from output file
c
c!Team-unique Header:
c
c!End
c-----------------------------------------------------------------------

      implicit none
      save

      
      include 'db_mod28uw_data.inc'
      include 'db_mod28uw_debug.inc'
      include 'platform_name.inc'
      
c ... arguments

      integer out_handle, scan, nscans, req_band, hdr_lun
      real array(max_pixel,max_line)
      character*(*) arrnm, b_unit_name
      logical init

c ... local variables
            
      integer i, j, out, ii, out_samples, out_lines, rtn, fil_datatype,
     &        fil_lun, fil_resolution, fil_offset, fil_samples, 
     &        fil_lines, fil_bands, out_maxsamples, out_maxlines,
     &        req_lines, req_samples, hdr_flg
      

      real fill_float, fil_error

c ... Local arrays
      byte out_flag(max_pixel*max_line)
      character*4 fil_interleave
      character*72 text
      character*10 fil_bandunits(OUT_BAND)
      character*80 fil_bandnames(OUT_BAND)
      character*256 fil_desc
      real float_data( max_pixel * max_line )

c ... external routines
      external db_write_flat_file
      integer db_write_flat_file

      integer string_length
      external string_length

      data init /.true./, ii /0/

      if (init) then
c ...   Set output file parameters
        req_samples = max_pixel
        req_lines = max_line

        fil_lun = out_handle
        fil_datatype = 4
        fil_interleave = 'bil'
        fil_resolution = 1
        fil_desc = 'Direct Broadcast Sea Surface Temperature Product'
        fil_error = -327.68
        fil_offset = 0
        fil_lines = nscans * max_line
        fil_bands = OUT_BAND
        fil_samples = max_pixel
        out_maxsamples = max_pixel
        out_maxlines = max_line
        init = .false.
      endif

c ... Fill in band and band unit information
      ii = ii + 1
      if (ii .le. OUT_BAND) then
        fil_bandnames(ii) = arrnm
        fil_bandunits(ii) = b_unit_name(1:string_length(b_unit_name))
      endif

c ... If we have gathered all the band names, then write header
      if (ii .eq. OUT_BAND) then
          hdr_flg = 1
      else
          hdr_flg = 0
      endif

      out = 1

c ... initialize the validity flag holder
      do j = 1 , max_line
        out_flag(out) = 0
        out = out + 1
      enddo

c ... Get the fill value
      
      fill_float = -327.68
      
c ... Create the scaled output array

      out = 1
      
      do j = 1, max_line

        do i = 1, max_pixel

c ...     Set default output value (fill value)
        
          float_data( out ) = fill_float

c ...     If array value is good, scale it and store it
        
          if ( abs( array( i, j ) - bad_value ) .gt. 1.0e-5 ) then

            float_data( out ) =  array( i, j ) 

          endif
          
          out = out + 1

        end do

      end do

c ... Write the output array and header to the output files

c ------ write the mask array to a flat output file
      rtn = db_write_flat_file(
     &                         scan,
     &                         req_samples,
     &                         req_lines,
     &                         req_band,
     &                         fil_lun,
     &                         hdr_flg,
     &                         hdr_lun,
     &                         platform_name,
     &                         fil_desc,
     &                         fil_datatype,
     &                         fil_interleave,
     &                         fil_resolution,
     &                         fil_error,
     &                         fil_offset,
     &                         fil_samples,
     &                         fil_lines,
     &                         fil_bands,
     &                         fil_bandnames,
     &                         fil_bandunits,
     &                         out_maxsamples,
     &                         out_maxlines,
     &                         float_data,
     &                         out_flag,
     &                         out_samples,
     &                         out_lines
     &                       )

      if ( rtn .ne. 0 ) then
        write(h_output, '(''Write failed on output array '',a)') arrnm
        write(h_output, '(5x,''rtn status = '',I9)') rtn
        write(text, '(''Write failed on output array '',a)') arrnm
        call message( 'db_mod28_write_products', text ,
     &   0, 2 )
      endif
                
      end


