#!/bin/csh -f
#
# This script is an example of how to set up processing for the 
# the IMAPP Level 2 MODIS Sea Surface Temperature Processing.
#
#  This script:
#  1) Copies over the binary ancillary input SST file
#  2) Extracts binary radiance/reflectance, geolocation and 
#     metadata information from MODIS DAAC or direct broadcast
#     HDF files.
#  3) Runs the IMAPP MODIS sea surface temperature software.
#
# Originally written by James E. Davies  January 2004
# Jim.Davies@ssec.wisc.edu

# Search for !!! for places where directoy locations are set and will
#  need to be changed for your on-site processing. 
#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Check arguments
if ($#argv != 4) then
  echo "Usage: modis_sst.csh SAT FIL1KM FILGEO OUTDIR"
  echo "where"
  echo "SAT    is the satellite name (Aqua or Terra)"
  echo "FIL1KM is the MODIS L1B 1000 meter resolution radiance flat file"
  echo "FILGEO is the MODIS L1B 1000 meter resolution geolocation flat file"
  echo "OUTDIR is the directory where the products will be stored"
  exit(-1)
endif

# Extract file names
set SAT    = $argv[1]
set FIL1KM = $argv[2]
set FILGEO = $argv[3]
set OUTDIR = $argv[4]

# Set root of output file name (e.g. 't1.02001.1815')
set ROOT = $FIL1KM:r
set ROOT = $ROOT:r
set ROOT = $ROOT:t

# Make sure Satellite name is recognizable
set fl=`echo $SAT|cut -c1`
if ( "$fl" == "T"  || "$fl" == "t") then
   set SAT = "Terra"
else if ( "$fl" == "A"  || "$fl" == "a" ) then
     set SAT = "Aqua"
else
     echo "Satellite name incorrect" $SAT
     exit 1
endif

#----------------------
# RUN THE SST ALGORITHM
#----------------------

# Print start message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started IMAPP MODIS Sea Surface Temperature processing at "`date`

#!!! Set directory for ancillary data !!!
set ANC_DIR = ../../data

# Find the SST file
set OISST_FILE =  `find $ANC_DIR -name "oisst.????????" | sort | tail -1`
set OISST = $OISST_FILE:t
/bin/cp ${ANC_DIR}/${OISST} ${OISST}

# Extract flat files
#echo "Started flatfile extraction at "`date`

#!!! Set directory for flatfile extraction data and src !!!
set SRCDIR = ../../flatfile/src
set DATA_DIR = ../../data

$SRCDIR/modis_extract_1km.exe $DATA_DIR/$FIL1KM $SAT ${ROOT}.1000m.img ${ROOT}.1000m.hdr
$SRCDIR/modis_extract_geo.exe $DATA_DIR/$FILGEO $SAT ${ROOT}.geo.img ${ROOT}.geo.hdr
echo "Finished flatfile extraction at "`date`
 
# Set SST output file names
set OUTSSTFIL   = $ROOT.mod28.img
set OUTSSTHDR   = $ROOT.mod28.hdr
set SSTDEBUG    = $ROOT.mod28debug.txt

# Create a configuration file from the template
#!!! Set sst src directory !!!
set SRCDIR = ../src
set TEMPLATE = $SRCDIR/sst.cfg
/bin/sed \
  -e "s/FIL1KM_IMG/${ROOT}.1000m.img/g" \
  -e "s/FIL1KM_HDR/${ROOT}.1000m.hdr/g" \
  -e "s/FILGEO_IMG/${ROOT}.geo.img/g" \
  -e "s/FILGEO_HDR/${ROOT}.geo.hdr/g" \
  -e "s/MOD28.img/${OUTSSTFIL}/g" \
  -e "s/MOD28.hdr/${OUTSSTHDR}/g" \
  -e "s/sstdebug.dat/${SSTDEBUG}/g" \
  -e "s/oisst_weekly/${OISST}/g" \
  $TEMPLATE > sst.cfg

# Run the SST
$SRCDIR/sst.exe sst.cfg $SAT

# Rename the configuration file
/bin/mv sst.cfg $ROOT.mod28.cfg

# Move the output files to the product directory
/bin/mv $OUTSSTFIL $OUTSSTHDR $SSTDEBUG $ROOT.mod28.cfg $OUTDIR

#-------------------------------------------------------------------------------
# CLEANUP AND EXIT
#-------------------------------------------------------------------------------

# Print end message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Finished IMAPP MODIS Sea Surface Temperature processing at "`date`

# Exit gracefully
exit(0)
